@@ -281,6 +281,8 @@ type (
 		TLS *auth.TLS `yaml:"tls"`
 		// Consistency configuration (defaults to LOCAL_QUORUM / LOCAL_SERIAL for all stores if this field not set)
 		Consistency *CassandraStoreConsistency `yaml:"consistency"`
+		// DisableInitialHostLookup instructs the gocql client to connect only using the supplied hosts
+		DisableInitialHostLookup bool `yaml:"disableInitialHostLookup"`
 	}
 
 	// CassandraStoreConsistency enables you to set the consistency settings for each Cassandra Persistence Store for Temporal
