@@ -31,6 +31,8 @@ import (
 	"sync/atomic"
 	"time"
 
+	"go.temporal.io/server/common/persistence/serialization"
+
 	"github.com/uber-go/tally"
 	"github.com/uber/tchannel-go"
 	"go.temporal.io/api/workflowservice/v1"
@@ -82,12 +84,13 @@ type (
 		metricsScope    tally.Scope
 		clusterMetadata cluster.Metadata
 		saProvider      searchattribute.Provider
+		saManager       searchattribute.Manager
 
 		// other common resources
 
 		namespaceCache    cache.NamespaceCache
 		timeSource        clock.TimeSource
-		payloadSerializer persistence.PayloadSerializer
+		payloadSerializer serialization.Serializer
 		metricsClient     metrics.Client
 		archivalMetadata  archiver.ArchivalMetadata
 		archiverProvider  provider.ArchiverProvider
@@ -203,8 +206,12 @@ func New(
 	}
 
 	dynamicCollection := dynamicconfig.NewCollection(params.DynamicConfigClient, logger)
+	factoryProvider := params.ClientFactoryProvider
+	if factoryProvider == nil {
+		factoryProvider = client.NewFactoryProvider()
+	}
 	clientBean, err := client.NewClientBean(
-		client.NewRPCClientFactory(
+		factoryProvider.NewFactory(
 			params.RPCFactory,
 			membershipMonitor,
 			params.MetricsClient,
@@ -240,6 +247,8 @@ func New(
 
 	saProvider := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
 
+	saManager := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
+
 	visibilityMgr, err := visibilityManagerInitializer(
 		persistenceBean,
 		saProvider,
@@ -312,12 +321,13 @@ func New(
 		metricsScope:    params.MetricsScope,
 		clusterMetadata: clusterMetadata,
 		saProvider:      saProvider,
+		saManager:       saManager,
 
 		// other common resources
 
 		namespaceCache:    namespaceCache,
 		timeSource:        clock.NewRealTimeSource(),
-		payloadSerializer: persistence.NewPayloadSerializer(),
+		payloadSerializer: serialization.NewSerializer(),
 		metricsClient:     params.MetricsClient,
 		archivalMetadata:  params.ArchivalMetadata,
 		archiverProvider:  params.ArchiverProvider,
@@ -452,7 +462,7 @@ func (h *Impl) GetTimeSource() clock.TimeSource {
 }
 
 // GetPayloadSerializer return binary payload serializer
-func (h *Impl) GetPayloadSerializer() persistence.PayloadSerializer {
+func (h *Impl) GetPayloadSerializer() serialization.Serializer {
 	return h.payloadSerializer
 }
 
@@ -626,3 +636,7 @@ func (h *Impl) GetGRPCListener() net.Listener {
 func (h *Impl) GetSearchAttributesProvider() searchattribute.Provider {
 	return h.saProvider
 }
+
+func (h *Impl) GetSearchAttributesManager() searchattribute.Manager {
+	return h.saManager
+}
