@@ -43,7 +43,7 @@ ES_PORT="${ES_PORT:-9200}"
 ES_USER="${ES_USER:-}"
 ES_PWD="${ES_PWD:-}"
 ES_VERSION="${ES_VERSION:-v6}"
-ES_VIS_INDEX="${ES_VIS_INDEX:-temporal-visibility-dev}"
+ES_VIS_INDEX="${ES_VIS_INDEX:-temporal_visibility_v1_dev}"
 ES_SCHEMA_SETUP_TIMEOUT_IN_SECONDS="${ES_SCHEMA_SETUP_TIMEOUT_IN_SECONDS:-0}"
 
 # Server setup
@@ -237,9 +237,9 @@ wait_for_es() {
 }
 
 setup_es_template() {
-    SCHEMA_FILE=${TEMPORAL_HOME}/schema/elasticsearch/${ES_VERSION}/visibility/index_template.json
+    SCHEMA_FILE=${TEMPORAL_HOME}/schema/elasticsearch/visibility/index_template_${ES_VERSION}.json
     ES_SERVER=$(echo "${ES_SEEDS}" | awk -F ',' '{print $1}')
-    TEMPLATE_URL="${ES_SCHEME}://${ES_SERVER}:${ES_PORT}/_template/temporal-visibility-template"
+    TEMPLATE_URL="${ES_SCHEME}://${ES_SERVER}:${ES_PORT}/_template/temporal_visibility_v1_template"
     INDEX_URL="${ES_SCHEME}://${ES_SERVER}:${ES_PORT}/${ES_VIS_INDEX}"
     curl --user "${ES_USER}":"${ES_PWD}" -X PUT "${TEMPLATE_URL}" -H 'Content-Type: application/json' --data-binary "@${SCHEMA_FILE}" --write-out "\n"
     curl --user "${ES_USER}":"${ES_PWD}" -X PUT "${INDEX_URL}" --write-out "\n"
@@ -259,18 +259,14 @@ register_default_namespace() {
 }
 
 add_custom_search_attributes() {
-    if ! tctl cluster get-search-attributes | grep CustomKeywordField; then
-        echo "Adding Custom*Field search attributes."
-        tctl --auto_confirm admin cluster add-search-attributes \
-            --name CustomKeywordField --type Keyword \
-            --name CustomStringField --type String \
-            --name CustomIntField --type Int \
-            --name CustomDatetimeField --type Datetime \
-            --name CustomDoubleField --type Double \
-            --name CustomBoolField --type Bool
-    else
-        echo "CustomKeywordField search attribute already exist. Skip search attributes creation."
-    fi
+      echo "Adding Custom*Field search attributes."
+      tctl --auto_confirm admin cluster add-search-attributes \
+          --name CustomKeywordField --type Keyword \
+          --name CustomStringField --type String \
+          --name CustomIntField --type Int \
+          --name CustomDatetimeField --type Datetime \
+          --name CustomDoubleField --type Double \
+          --name CustomBoolField --type Bool
 }
 
 setup_server(){
