@@ -30,6 +30,8 @@ import (
 	"fmt"
 	"sync/atomic"
 
+	"go.temporal.io/server/common/persistence/serialization"
+
 	"github.com/pborman/uuid"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
@@ -78,7 +80,7 @@ type (
 		ESClient              esclient.Client
 		config                *Config
 		namespaceDLQHandler   namespace.DLQMessageHandler
-		eventSerializer       persistence.PayloadSerializer
+		eventSerializer       serialization.Serializer
 	}
 )
 
@@ -110,7 +112,7 @@ func NewAdminHandler(
 			resource.GetNamespaceReplicationQueue(),
 			resource.GetLogger(),
 		),
-		eventSerializer: persistence.NewPayloadSerializer(),
+		eventSerializer: serialization.NewSerializer(),
 		ESConfig:        params.ESConfig,
 		ESClient:        params.ESClient,
 	}
@@ -172,7 +174,7 @@ func (adh *AdminHandler) AddSearchAttributes(ctx context.Context, request *admin
 	}
 
 	for saName, saType := range request.GetSearchAttributes() {
-		if searchattribute.IsReservedField(saName) {
+		if searchattribute.IsReserved(saName) {
 			return nil, adh.error(serviceerror.NewInvalidArgument(fmt.Sprintf(errSearchAttributeIsReservedMessage, saName)), scope)
 		}
 		if currentSearchAttributes.IsDefined(saName) {
@@ -210,21 +212,56 @@ func (adh *AdminHandler) AddSearchAttributes(ctx context.Context, request *admin
 	}
 	scope.IncCounter(metrics.AddSearchAttributesWorkflowSuccessCount)
 
-	// Get current state and return it.
-	resp, err := adh.getSearchAttributes(ctx, indexName, run.GetRunID())
-	if err != nil {
-		// Don't return error in case of getSearchAttributes error. Just log it.
-		adh.GetLogger().Error("Unable to get search attributes back while adding them.", tag.Error(err))
-		metricsGetSearchAttributesScope := adh.GetMetricsClient().Scope(metrics.AdminGetSearchAttributesScope)
-		metricsGetSearchAttributesScope.IncCounter(metrics.ServiceFailures)
+	return &adminservice.AddSearchAttributesResponse{}, nil
+}
+
+// RemoveSearchAttributes remove search attribute from the cluster.
+func (adh *AdminHandler) RemoveSearchAttributes(ctx context.Context, request *adminservice.RemoveSearchAttributesRequest) (_ *adminservice.RemoveSearchAttributesResponse, retError error) {
+	defer log.CapturePanic(adh.GetLogger(), &retError)
+
+	scope, sw := adh.startRequestProfile(metrics.AdminRemoveSearchAttributesScope)
+	defer sw.Stop()
+
+	// validate request
+	if request == nil {
+		return nil, adh.error(errRequestNotSet, scope)
 	}
 
-	return &adminservice.AddSearchAttributesResponse{
-		CustomAttributes:         resp.GetCustomAttributes(),
-		SystemAttributes:         resp.GetSystemAttributes(),
-		Mapping:                  resp.GetMapping(),
-		AddWorkflowExecutionInfo: resp.GetAddWorkflowExecutionInfo(),
-	}, nil
+	if len(request.GetSearchAttributes()) == 0 {
+		return nil, adh.error(errSearchAttributesNotSet, scope)
+	}
+
+	indexName := request.GetIndexName()
+	if indexName == "" {
+		indexName = adh.ESConfig.GetVisibilityIndex()
+	}
+
+	currentSearchAttributes, err := adh.Resource.GetSearchAttributesProvider().GetSearchAttributes(indexName, true)
+	if err != nil {
+		return nil, adh.error(serviceerror.NewInternal(fmt.Sprintf(errUnableToGetSearchAttributesMessage, err)), scope)
+	}
+
+	newCustomSearchAttributes := map[string]enumspb.IndexedValueType{}
+	for saName, saType := range currentSearchAttributes.Custom() {
+		newCustomSearchAttributes[saName] = saType
+	}
+
+	for _, saName := range request.GetSearchAttributes() {
+		if !currentSearchAttributes.IsDefined(saName) {
+			return nil, adh.error(serviceerror.NewInvalidArgument(fmt.Sprintf(errSearchAttributeDoesntExistMessage, saName)), scope)
+		}
+		if _, ok := newCustomSearchAttributes[saName]; !ok {
+			return nil, adh.error(serviceerror.NewInvalidArgument(fmt.Sprintf(errUnableToRemoveNonCustomSearchAttributesMessage, saName)), scope)
+		}
+		delete(newCustomSearchAttributes, saName)
+	}
+
+	err = adh.Resource.GetSearchAttributesManager().SaveSearchAttributes(indexName, newCustomSearchAttributes)
+	if err != nil {
+		return nil, adh.error(serviceerror.NewInternal(fmt.Sprintf(errUnableToSaveSearchAttributesMessage, err)), scope)
+	}
+
+	return &adminservice.RemoveSearchAttributesResponse{}, nil
 }
 
 func (adh *AdminHandler) GetSearchAttributes(ctx context.Context, request *adminservice.GetSearchAttributesRequest) (_ *adminservice.GetSearchAttributesResponse, retError error) {
@@ -233,6 +270,10 @@ func (adh *AdminHandler) GetSearchAttributes(ctx context.Context, request *admin
 	scope, sw := adh.startRequestProfile(metrics.AdminGetSearchAttributesScope)
 	defer sw.Stop()
 
+	if request == nil {
+		return nil, adh.error(errRequestNotSet, scope)
+	}
+
 	indexName := request.GetIndexName()
 	if indexName == "" {
 		indexName = adh.ESConfig.GetVisibilityIndex()
