@@ -391,7 +391,7 @@ func (s *adminHandlerSuite) Test_SetRequestDefaultValueAndGetTargetVersionHistor
 	s.NoError(err)
 }
 
-func (s *adminHandlerSuite) Test_AddSearchAttribute_Validate() {
+func (s *adminHandlerSuite) Test_AddSearchAttributes() {
 	handler := s.handler
 	ctx := context.Background()
 
@@ -454,11 +454,11 @@ func (s *adminHandlerSuite) Test_AddSearchAttribute_Validate() {
 	// Configure Elasticsearch: add advanced visibility store config with index name.
 	handler.ESConfig = &config.Elasticsearch{
 		Indices: map[string]string{
-			"visibility": "temporal-visibility-dev",
+			"visibility": "random-index-name",
 		},
 	}
 
-	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("temporal-visibility-dev", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("random-index-name", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
 	testCases2 := []test{
 		{
 			Name: "reserved key (ES configured)",
@@ -515,10 +515,6 @@ func (s *adminHandlerSuite) Test_AddSearchAttribute_Validate() {
 	mockRun.On("Get", mock.Anything, nil).Return(nil)
 	mockRun.On("GetRunID").Return("random-run-id")
 
-	s.mockResource.SDKClient.On("DescribeWorkflowExecution", mock.Anything, "temporal-sys-add-search-attributes-workflow", "random-run-id").Return(
-		&workflowservice.DescribeWorkflowExecutionResponse{}, nil)
-	s.mockResource.ESClient.EXPECT().GetMapping(gomock.Any(), "temporal-visibility-dev").Return(map[string]string{"col": "type"}, nil)
-
 	resp, err = handler.AddSearchAttributes(ctx, &adminservice.AddSearchAttributesRequest{
 		SearchAttributes: map[string]enumspb.IndexedValueType{
 			"CustomAttr": enumspb.INDEXED_VALUE_TYPE_KEYWORD,
@@ -527,3 +523,162 @@ func (s *adminHandlerSuite) Test_AddSearchAttribute_Validate() {
 	s.NoError(err)
 	s.NotNil(resp)
 }
+
+func (s *adminHandlerSuite) Test_GetSearchAttributes() {
+	handler := s.handler
+	ctx := context.Background()
+
+	resp, err := handler.GetSearchAttributes(ctx, nil)
+	s.Error(err)
+	s.Equal(&serviceerror.InvalidArgument{Message: "Request is nil."}, err)
+	s.Nil(resp)
+
+	// Elasticsearch is not configured
+	s.mockResource.SDKClient.On("DescribeWorkflowExecution", mock.Anything, "temporal-sys-add-search-attributes-workflow", "").Return(
+		&workflowservice.DescribeWorkflowExecutionResponse{}, nil).Once()
+	s.mockResource.ESClient.EXPECT().GetMapping(gomock.Any(), "").Return(map[string]string{"col": "type"}, nil)
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+
+	resp, err = handler.GetSearchAttributes(ctx, &adminservice.GetSearchAttributesRequest{})
+	s.NoError(err)
+	s.NotNil(resp)
+
+	// Configure Elasticsearch: add advanced visibility store config with index name.
+	handler.ESConfig = &config.Elasticsearch{
+		Indices: map[string]string{
+			"visibility": "random-index-name",
+		},
+	}
+
+	s.mockResource.SDKClient.On("DescribeWorkflowExecution", mock.Anything, "temporal-sys-add-search-attributes-workflow", "").Return(
+		&workflowservice.DescribeWorkflowExecutionResponse{}, nil).Once()
+	s.mockResource.ESClient.EXPECT().GetMapping(gomock.Any(), "random-index-name").Return(map[string]string{"col": "type"}, nil)
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("random-index-name", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	resp, err = handler.GetSearchAttributes(ctx, &adminservice.GetSearchAttributesRequest{})
+	s.NoError(err)
+	s.NotNil(resp)
+
+	s.mockResource.SDKClient.On("DescribeWorkflowExecution", mock.Anything, "temporal-sys-add-search-attributes-workflow", "").Return(
+		&workflowservice.DescribeWorkflowExecutionResponse{}, nil).Once()
+	s.mockResource.ESClient.EXPECT().GetMapping(gomock.Any(), "another-index-name").Return(map[string]string{"col": "type"}, nil)
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("another-index-name", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	resp, err = handler.GetSearchAttributes(ctx, &adminservice.GetSearchAttributesRequest{IndexName: "another-index-name"})
+	s.NoError(err)
+	s.NotNil(resp)
+
+	s.mockResource.SDKClient.On("DescribeWorkflowExecution", mock.Anything, "temporal-sys-add-search-attributes-workflow", "").Return(
+		nil, errors.New("random error")).Once()
+	s.mockResource.ESClient.EXPECT().GetMapping(gomock.Any(), "random-index-name").Return(map[string]string{"col": "type"}, nil)
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("random-index-name", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	resp, err = handler.GetSearchAttributes(ctx, &adminservice.GetSearchAttributesRequest{})
+	s.Error(err)
+	s.Nil(resp)
+}
+
+func (s *adminHandlerSuite) Test_RemoveSearchAttributes() {
+	handler := s.handler
+	ctx := context.Background()
+
+	type test struct {
+		Name     string
+		Request  *adminservice.RemoveSearchAttributesRequest
+		Expected error
+	}
+	// request validation tests
+	testCases1 := []test{
+		{
+			Name:     "nil request",
+			Request:  nil,
+			Expected: &serviceerror.InvalidArgument{Message: "Request is nil."},
+		},
+		{
+			Name:     "empty request",
+			Request:  &adminservice.RemoveSearchAttributesRequest{},
+			Expected: &serviceerror.InvalidArgument{Message: "SearchAttributes are not set on request."},
+		},
+	}
+	for _, testCase := range testCases1 {
+		s.T().Run(testCase.Name, func(t *testing.T) {
+			resp, err := handler.RemoveSearchAttributes(ctx, testCase.Request)
+			s.Equal(testCase.Expected, err)
+			s.Nil(resp)
+		})
+	}
+
+	// Elasticsearch is not configured
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	testCases3 := []test{
+		{
+			Name: "reserved search attribute (empty index)",
+			Request: &adminservice.RemoveSearchAttributesRequest{
+				SearchAttributes: []string{
+					"WorkflowId",
+				},
+			},
+			Expected: &serviceerror.InvalidArgument{Message: "Unable to remove non-custom search attributes: WorkflowId."},
+		},
+		{
+			Name: "search attribute doesn't exist (empty index)",
+			Request: &adminservice.RemoveSearchAttributesRequest{
+				SearchAttributes: []string{
+					"ProductId",
+				},
+			},
+			Expected: &serviceerror.InvalidArgument{Message: "Search attribute ProductId doesn't exist."},
+		},
+	}
+	for _, testCase := range testCases3 {
+		s.T().Run(testCase.Name, func(t *testing.T) {
+			resp, err := handler.RemoveSearchAttributes(ctx, testCase.Request)
+			s.Equal(testCase.Expected, err)
+			s.Nil(resp)
+		})
+	}
+
+	// Configure Elasticsearch: add advanced visibility store config with index name.
+	handler.ESConfig = &config.Elasticsearch{
+		Indices: map[string]string{
+			"visibility": "random-index-name",
+		},
+	}
+
+	s.mockResource.SearchAttributesProvider.EXPECT().GetSearchAttributes("random-index-name", true).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+	testCases2 := []test{
+		{
+			Name: "reserved search attribute (ES configured)",
+			Request: &adminservice.RemoveSearchAttributesRequest{
+				SearchAttributes: []string{
+					"WorkflowId",
+				},
+			},
+			Expected: &serviceerror.InvalidArgument{Message: "Unable to remove non-custom search attributes: WorkflowId."},
+		},
+		{
+			Name: "search attribute doesn't exist (ES configured)",
+			Request: &adminservice.RemoveSearchAttributesRequest{
+				SearchAttributes: []string{
+					"ProductId",
+				},
+			},
+			Expected: &serviceerror.InvalidArgument{Message: "Search attribute ProductId doesn't exist."},
+		},
+	}
+	for _, testCase := range testCases2 {
+		s.T().Run(testCase.Name, func(t *testing.T) {
+			resp, err := handler.RemoveSearchAttributes(ctx, testCase.Request)
+			s.Equal(testCase.Expected, err)
+			s.Nil(resp)
+		})
+	}
+
+	// Success case.
+	s.mockResource.SearchAttributesManager.EXPECT().SaveSearchAttributes("random-index-name", gomock.Any()).Return(nil)
+
+	resp, err := handler.RemoveSearchAttributes(ctx, &adminservice.RemoveSearchAttributesRequest{
+		SearchAttributes: []string{
+			"CustomKeywordField",
+		},
+	})
+	s.NoError(err)
+	s.NotNil(resp)
+}
