@@ -28,6 +28,8 @@ import (
 	"context"
 	"time"
 
+	"go.temporal.io/server/common/persistence/serialization"
+
 	"github.com/pborman/uuid"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
@@ -45,6 +47,7 @@ import (
 	"go.temporal.io/server/common/primitives/timestamp"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/workflow"
 )
 
 var (
@@ -59,24 +62,25 @@ const (
 
 type (
 	stateBuilderProvider func(
-		mutableState mutableState,
-		logger log.Logger) stateBuilder
+		mutableState workflow.MutableState,
+		logger log.Logger,
+	) workflow.MutableStateRebuilder
 
 	mutableStateProvider func(
 		namespaceEntry *cache.NamespaceCacheEntry,
 		startTime time.Time,
 		logger log.Logger,
-	) mutableState
+	) workflow.MutableState
 
 	nDCBranchMgrProvider func(
-		context workflowExecutionContext,
-		mutableState mutableState,
+		context workflow.Context,
+		mutableState workflow.MutableState,
 		logger log.Logger,
 	) nDCBranchMgr
 
 	nDCConflictResolverProvider func(
-		context workflowExecutionContext,
-		mutableState mutableState,
+		context workflow.Context,
+		mutableState workflow.MutableState,
 		logger log.Logger,
 	) nDCConflictResolver
 
@@ -84,7 +88,7 @@ type (
 		namespaceID string,
 		workflowID string,
 		baseRunID string,
-		newContext workflowExecutionContext,
+		newContext workflow.Context,
 		newRunID string,
 		logger log.Logger,
 	) nDCWorkflowResetter
@@ -100,10 +104,10 @@ type (
 		shard             shard.Context
 		clusterMetadata   cluster.Metadata
 		historyV2Mgr      persistence.HistoryManager
-		historySerializer persistence.PayloadSerializer
+		historySerializer serialization.Serializer
 		metricsClient     metrics.Client
 		namespaceCache    cache.NamespaceCache
-		historyCache      *historyCache
+		historyCache      *workflow.Cache
 		eventsReapplier   nDCEventsReapplier
 		transactionMgr    nDCTransactionMgr
 		logger            log.Logger
@@ -120,7 +124,7 @@ var errPanic = serviceerror.NewInternal("encounter panic")
 
 func newNDCHistoryReplicator(
 	shard shard.Context,
-	historyCache *historyCache,
+	historyCache *workflow.Cache,
 	eventsReapplier nDCEventsReapplier,
 	logger log.Logger,
 ) *nDCHistoryReplicatorImpl {
@@ -130,7 +134,7 @@ func newNDCHistoryReplicator(
 		shard:             shard,
 		clusterMetadata:   shard.GetService().GetClusterMetadata(),
 		historyV2Mgr:      shard.GetHistoryManager(),
-		historySerializer: persistence.NewPayloadSerializer(),
+		historySerializer: serialization.NewSerializer(),
 		metricsClient:     shard.GetMetricsClient(),
 		namespaceCache:    shard.GetNamespaceCache(),
 		historyCache:      historyCache,
@@ -139,15 +143,15 @@ func newNDCHistoryReplicator(
 		logger:            log.With(logger, tag.ComponentHistoryReplicator),
 
 		newBranchMgr: func(
-			context workflowExecutionContext,
-			mutableState mutableState,
+			context workflow.Context,
+			mutableState workflow.MutableState,
 			logger log.Logger,
 		) nDCBranchMgr {
 			return newNDCBranchMgr(shard, context, mutableState, logger)
 		},
 		newConflictResolver: func(
-			context workflowExecutionContext,
-			mutableState mutableState,
+			context workflow.Context,
+			mutableState workflow.MutableState,
 			logger log.Logger,
 		) nDCConflictResolver {
 			return newNDCConflictResolver(shard, context, mutableState, logger)
@@ -156,23 +160,23 @@ func newNDCHistoryReplicator(
 			namespaceID string,
 			workflowID string,
 			baseRunID string,
-			newContext workflowExecutionContext,
+			newContext workflow.Context,
 			newRunID string,
 			logger log.Logger,
 		) nDCWorkflowResetter {
 			return newNDCWorkflowResetter(shard, transactionMgr, namespaceID, workflowID, baseRunID, newContext, newRunID, logger)
 		},
 		newStateBuilder: func(
-			state mutableState,
+			state workflow.MutableState,
 			logger log.Logger,
-		) stateBuilder {
+		) workflow.MutableStateRebuilder {
 
-			return newStateBuilder(
+			return workflow.NewMutableStateRebuilder(
 				shard,
 				logger,
 				state,
-				func(mutableState mutableState) mutableStateTaskGenerator {
-					return newMutableStateTaskGenerator(shard.GetNamespaceCache(), logger, mutableState)
+				func(mutableState workflow.MutableState) workflow.TaskGenerator {
+					return workflow.NewTaskGenerator(shard.GetNamespaceCache(), logger, mutableState)
 				},
 			)
 		},
@@ -180,8 +184,8 @@ func newNDCHistoryReplicator(
 			namespaceEntry *cache.NamespaceCacheEntry,
 			startTime time.Time,
 			logger log.Logger,
-		) mutableState {
-			return newMutableStateBuilder(
+		) workflow.MutableState {
+			return workflow.NewMutableState(
 				shard,
 				shard.GetEventsCache(),
 				logger,
@@ -219,10 +223,11 @@ func (r *nDCHistoryReplicatorImpl) applyEvents(
 	task nDCReplicationTask,
 ) (retError error) {
 
-	context, releaseFn, err := r.historyCache.getOrCreateWorkflowExecution(
+	context, releaseFn, err := r.historyCache.GetOrCreateWorkflowExecution(
 		ctx,
 		task.getNamespaceID(),
 		*task.getExecution(),
+		workflow.CallerTypeAPI,
 	)
 	if err != nil {
 		// for get workflow execution context, with valid run id
@@ -245,9 +250,9 @@ func (r *nDCHistoryReplicatorImpl) applyEvents(
 	default:
 		// apply events, other than simple start workflow execution
 		// the continue as new + start workflow execution combination will also be processed here
-		var mutableState mutableState
+		var mutableState workflow.MutableState
 		var err error
-		namespaceEntry, err := r.namespaceCache.GetNamespaceByID(context.getNamespaceID())
+		namespaceEntry, err := r.namespaceCache.GetNamespaceByID(context.GetNamespaceID())
 		if err != nil {
 			return err
 		}
@@ -255,9 +260,9 @@ func (r *nDCHistoryReplicatorImpl) applyEvents(
 		if r.shard.GetConfig().ReplicationEventsFromCurrentCluster(namespaceEntry.GetInfo().Name) {
 			// this branch is used when replicating events (generated from current cluster)from remote cluster to current cluster.
 			// this could happen when the events are lost in current cluster and plan to recover them from remote cluster.
-			mutableState, err = context.loadWorkflowExecutionForReplication(task.getVersion())
+			mutableState, err = context.LoadWorkflowExecutionForReplication(task.getVersion())
 		} else {
-			mutableState, err = context.loadWorkflowExecution()
+			mutableState, err = context.LoadWorkflowExecution()
 		}
 		switch err.(type) {
 		case nil:
@@ -302,8 +307,8 @@ func (r *nDCHistoryReplicatorImpl) applyEvents(
 
 func (r *nDCHistoryReplicatorImpl) applyStartEvents(
 	ctx context.Context,
-	context workflowExecutionContext,
-	releaseFn releaseWorkflowExecutionFunc,
+	context workflow.Context,
+	releaseFn workflow.ReleaseCacheFunc,
 	task nDCReplicationTask,
 ) (retError error) {
 
@@ -316,7 +321,7 @@ func (r *nDCHistoryReplicatorImpl) applyStartEvents(
 	stateBuilder := r.newStateBuilder(mutableState, task.getLogger())
 
 	// use state builder for workflow mutable state mutation
-	_, err = stateBuilder.applyEvents(
+	_, err = stateBuilder.ApplyEvents(
 		task.getNamespaceID(),
 		requestID,
 		*task.getExecution(),
@@ -356,8 +361,8 @@ func (r *nDCHistoryReplicatorImpl) applyStartEvents(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsPrepareBranch(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	task nDCReplicationTask,
 ) (bool, int32, error) {
 
@@ -387,11 +392,11 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsPrepareBranch(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsPrepareMutableState(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	branchIndex int32,
 	task nDCReplicationTask,
-) (mutableState, bool, error) {
+) (workflow.MutableState, bool, error) {
 
 	incomingVersion := task.getVersion()
 	conflictResolver := r.newConflictResolver(context, mutableState, task.getLogger())
@@ -411,16 +416,16 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsPrepareMutableState(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	isRebuilt bool,
-	releaseFn releaseWorkflowExecutionFunc,
+	releaseFn workflow.ReleaseCacheFunc,
 	task nDCReplicationTask,
 ) error {
 
 	requestID := uuid.New() // requestID used for start workflow execution request.  This is not on the history event.
 	stateBuilder := r.newStateBuilder(mutableState, task.getLogger())
-	newMutableState, err := stateBuilder.applyEvents(
+	newMutableState, err := stateBuilder.ApplyEvents(
 		task.getNamespaceID(),
 		requestID,
 		*task.getExecution(),
@@ -448,7 +453,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 	if newMutableState != nil {
 		newExecutionInfo := newMutableState.GetExecutionInfo()
 		newExecutionState := newMutableState.GetExecutionState()
-		newContext := newWorkflowExecutionContext(
+		newContext := workflow.NewContext(
 			newExecutionInfo.NamespaceId,
 			commonpb.WorkflowExecution{
 				WorkflowId: newExecutionInfo.WorkflowId,
@@ -465,7 +470,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 			r.clusterMetadata,
 			newContext,
 			newMutableState,
-			noopReleaseFn,
+			workflow.NoopReleaseFn,
 		)
 	}
 
@@ -489,10 +494,10 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToCurrentBranch(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranch(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	branchIndex int32,
-	releaseFn releaseWorkflowExecutionFunc,
+	releaseFn workflow.ReleaseCacheFunc,
 	task nDCReplicationTask,
 ) error {
 
@@ -517,10 +522,10 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranch(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithoutContinueAsNew(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	branchIndex int32,
-	releaseFn releaseWorkflowExecutionFunc,
+	releaseFn workflow.ReleaseCacheFunc,
 	task nDCReplicationTask,
 ) error {
 
@@ -574,8 +579,8 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithout
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithContinueAsNew(
 	ctx context.Context,
-	context workflowExecutionContext,
-	releaseFn releaseWorkflowExecutionFunc,
+	context workflow.Context,
+	releaseFn workflow.ReleaseCacheFunc,
 	task nDCReplicationTask,
 ) error {
 
@@ -584,12 +589,12 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithCon
 	// NOTE: need to release target workflow due to target workflow
 	//  can potentially be the current workflow causing deadlock
 
-	// 1. clear all in memory changes & release target workflow lock
+	// 1. clear all in memory changes & release target workflow Lock
 	// 2. apply new workflow first
 	// 3. apply target workflow
 
 	// step 1
-	context.clear()
+	context.Clear()
 	releaseFn(nil)
 
 	// step 2
@@ -619,9 +624,9 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsToNoneCurrentBranchWithCon
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsMissingMutableState(
 	ctx context.Context,
-	newContext workflowExecutionContext,
+	newContext workflow.Context,
 	task nDCReplicationTask,
-) (mutableState, error) {
+) (workflow.MutableState, error) {
 
 	// for non reset workflow execution replication task, just do re-replication
 	if !task.isWorkflowReset() {
@@ -674,14 +679,14 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsMissingMutableState(
 
 func (r *nDCHistoryReplicatorImpl) applyNonStartEventsResetWorkflow(
 	ctx context.Context,
-	context workflowExecutionContext,
-	mutableState mutableState,
+	context workflow.Context,
+	mutableState workflow.MutableState,
 	task nDCReplicationTask,
 ) error {
 
 	requestID := uuid.New() // requestID used for start workflow execution request.  This is not on the history event.
 	stateBuilder := r.newStateBuilder(mutableState, task.getLogger())
-	_, err := stateBuilder.applyEvents(
+	_, err := stateBuilder.ApplyEvents(
 		task.getNamespaceID(),
 		requestID,
 		*task.getExecution(),
@@ -702,7 +707,7 @@ func (r *nDCHistoryReplicatorImpl) applyNonStartEventsResetWorkflow(
 		r.clusterMetadata,
 		context,
 		mutableState,
-		noopReleaseFn,
+		workflow.NoopReleaseFn,
 	)
 
 	err = r.transactionMgr.createWorkflow(
