@@ -52,7 +52,9 @@ import (
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
+	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/workflow"
 	"go.temporal.io/server/service/worker/parentclosepolicy"
 )
 
@@ -128,8 +130,14 @@ func (t *transferQueueActiveTaskExecutor) processActivityTask(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	context, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	context, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -167,8 +175,14 @@ func (t *transferQueueActiveTaskExecutor) processWorkflowTask(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	context, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	context, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -230,8 +244,14 @@ func (t *transferQueueActiveTaskExecutor) processCloseExecution(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	weContext, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	weContext, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -275,12 +295,8 @@ func (t *transferQueueActiveTaskExecutor) processCloseExecution(
 	workflowHistoryLength := mutableState.GetNextEventID() - 1
 	taskQueue := executionInfo.TaskQueue
 
-	startEvent, err := mutableState.GetStartEvent()
-	if err != nil {
-		return err
-	}
-	workflowStartTime := timestamp.TimeValue(startEvent.GetEventTime())
-	workflowExecutionTime := getWorkflowExecutionTime(mutableState, startEvent)
+	workflowStartTime := timestamp.TimeValue(mutableState.GetExecutionInfo().GetStartTime())
+	workflowExecutionTime := timestamp.TimeValue(mutableState.GetExecutionInfo().GetExecutionTime())
 	visibilityMemo := getWorkflowMemo(copyMemo(executionInfo.Memo))
 	searchAttr := getSearchAttributes(copySearchAttributes(executionInfo.SearchAttributes))
 	namespace := mutableState.GetNamespaceEntry().GetInfo().Name
@@ -343,8 +359,14 @@ func (t *transferQueueActiveTaskExecutor) processCancelExecution(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	context, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	context, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -427,8 +449,14 @@ func (t *transferQueueActiveTaskExecutor) processSignalExecution(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	weContext, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	weContext, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -521,7 +549,7 @@ func (t *transferQueueActiveTaskExecutor) processSignalExecution(
 	// the rest of logic is making RPC call, which takes time.
 	release(retError)
 	// remove signalRequestedID from target workflow, after Signal detail is removed from source workflow
-	ctx, cancel := context.WithTimeout(context.Background(), transferActiveTaskDefaultTimeout)
+	ctx, cancel = context.WithTimeout(context.Background(), taskTimeout)
 	defer cancel()
 	_, err = t.historyClient.RemoveSignalMutableState(ctx, &historyservice.RemoveSignalMutableStateRequest{
 		NamespaceId: task.GetTargetNamespaceId(),
@@ -538,8 +566,14 @@ func (t *transferQueueActiveTaskExecutor) processStartChildExecution(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	context, release, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	context, release, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -649,8 +683,14 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 	task *persistencespb.TransferTaskInfo,
 ) (retError error) {
 
-	currentContext, currentRelease, err := t.cache.getOrCreateWorkflowExecutionForBackground(
-		t.getNamespaceIDAndWorkflowExecution(task),
+	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+	defer cancel()
+	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(task)
+	currentContext, currentRelease, err := t.cache.GetOrCreateWorkflowExecution(
+		ctx,
+		namespaceID,
+		execution,
+		workflow.CallerTypeTask,
 	)
 	if err != nil {
 		return err
@@ -710,7 +750,7 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 	}
 	logger = log.With(logger, tag.WorkflowNamespace(namespaceEntry.GetInfo().Name))
 
-	reason, resetPoint := FindAutoResetPoint(t.shard.GetTimeSource(), namespaceEntry.GetConfig().BadBinaries, executionInfo.AutoResetPoints)
+	reason, resetPoint := workflow.FindAutoResetPoint(t.shard.GetTimeSource(), namespaceEntry.GetConfig().BadBinaries, executionInfo.AutoResetPoints)
 	if resetPoint == nil {
 		logger.Warn("Auto-Reset is skipped, because reset point is not found.")
 		return nil
@@ -722,9 +762,9 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 		tag.WorkflowEventID(resetPoint.GetFirstWorkflowTaskCompletedId()),
 	)
 
-	var baseContext workflowExecutionContext
-	var baseMutableState mutableState
-	var baseRelease releaseWorkflowExecutionFunc
+	var baseContext workflow.Context
+	var baseMutableState workflow.MutableState
+	var baseRelease workflow.ReleaseCacheFunc
 	if resetPoint.GetRunId() == executionState.RunId {
 		baseContext = currentContext
 		baseMutableState = currentMutableState
@@ -734,7 +774,14 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 			WorkflowId: task.GetWorkflowId(),
 			RunId:      resetPoint.GetRunId(),
 		}
-		baseContext, baseRelease, err = t.cache.getOrCreateWorkflowExecutionForBackground(task.GetNamespaceId(), *baseExecution)
+		ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
+		defer cancel()
+		baseContext, baseRelease, err = t.cache.GetOrCreateWorkflowExecution(
+			ctx,
+			task.GetNamespaceId(),
+			*baseExecution,
+			workflow.CallerTypeTask,
+		)
 		if err != nil {
 			return err
 		}
@@ -766,13 +813,13 @@ func (t *transferQueueActiveTaskExecutor) processResetWorkflow(
 
 func (t *transferQueueActiveTaskExecutor) recordChildExecutionStarted(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	initiatedAttributes *historypb.StartChildWorkflowExecutionInitiatedEventAttributes,
 	runID string,
 ) error {
 
 	return t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return serviceerror.NewNotFound("Workflow execution already completed.")
 			}
@@ -801,12 +848,12 @@ func (t *transferQueueActiveTaskExecutor) recordChildExecutionStarted(
 
 func (t *transferQueueActiveTaskExecutor) recordStartChildExecutionFailed(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	initiatedAttributes *historypb.StartChildWorkflowExecutionInitiatedEventAttributes,
 ) error {
 
 	return t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return serviceerror.NewNotFound("Workflow execution already completed.")
 			}
@@ -854,14 +901,14 @@ func (t *transferQueueActiveTaskExecutor) createFirstWorkflowTask(
 
 func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionCompleted(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	targetNamespace string,
 	targetWorkflowID string,
 	targetRunID string,
 ) error {
 
 	err := t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return &serviceerror.NotFound{Message: "Workflow execution already completed."}
 			}
@@ -869,7 +916,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionComplete
 			initiatedEventID := task.GetScheduleId()
 			_, ok := mutableState.GetRequestCancelInfo(initiatedEventID)
 			if !ok {
-				return ErrMissingRequestCancelInfo
+				return workflow.ErrMissingRequestCancelInfo
 			}
 
 			_, err := mutableState.AddExternalWorkflowExecutionCancelRequested(
@@ -891,7 +938,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionComplete
 
 func (t *transferQueueActiveTaskExecutor) signalExternalExecutionCompleted(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	targetNamespace string,
 	targetWorkflowID string,
 	targetRunID string,
@@ -899,7 +946,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionCompleted(
 ) error {
 
 	err := t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return &serviceerror.NotFound{Message: "Workflow execution already completed."}
 			}
@@ -907,7 +954,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionCompleted(
 			initiatedEventID := task.GetScheduleId()
 			_, ok := mutableState.GetSignalInfo(initiatedEventID)
 			if !ok {
-				return ErrMissingSignalInfo
+				return workflow.ErrMissingSignalInfo
 			}
 
 			_, err := mutableState.AddExternalWorkflowExecutionSignaled(
@@ -930,14 +977,14 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionCompleted(
 
 func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionFailed(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	targetNamespace string,
 	targetWorkflowID string,
 	targetRunID string,
 ) error {
 
 	err := t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return &serviceerror.NotFound{Message: "Workflow execution already completed."}
 			}
@@ -945,7 +992,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionFailed(
 			initiatedEventID := task.GetScheduleId()
 			_, ok := mutableState.GetRequestCancelInfo(initiatedEventID)
 			if !ok {
-				return ErrMissingRequestCancelInfo
+				return workflow.ErrMissingRequestCancelInfo
 			}
 
 			_, err := mutableState.AddRequestCancelExternalWorkflowExecutionFailedEvent(
@@ -968,7 +1015,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionFailed(
 
 func (t *transferQueueActiveTaskExecutor) signalExternalExecutionFailed(
 	task *persistencespb.TransferTaskInfo,
-	context workflowExecutionContext,
+	context workflow.Context,
 	targetNamespace string,
 	targetWorkflowID string,
 	targetRunID string,
@@ -976,7 +1023,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionFailed(
 ) error {
 
 	err := t.updateWorkflowExecution(context, true,
-		func(mutableState mutableState) error {
+		func(mutableState workflow.MutableState) error {
 			if !mutableState.IsWorkflowExecutionRunning() {
 				return &serviceerror.NotFound{Message: "Workflow is not running."}
 			}
@@ -984,7 +1031,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionFailed(
 			initiatedEventID := task.GetScheduleId()
 			_, ok := mutableState.GetSignalInfo(initiatedEventID)
 			if !ok {
-				return ErrMissingSignalInfo
+				return workflow.ErrMissingSignalInfo
 			}
 
 			_, err := mutableState.AddSignalExternalWorkflowExecutionFailedEvent(
@@ -1007,12 +1054,12 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionFailed(
 }
 
 func (t *transferQueueActiveTaskExecutor) updateWorkflowExecution(
-	context workflowExecutionContext,
+	context workflow.Context,
 	createWorkflowTask bool,
-	action func(builder mutableState) error,
+	action func(builder workflow.MutableState) error,
 ) error {
 
-	mutableState, err := context.loadWorkflowExecution()
+	mutableState, err := context.LoadWorkflowExecution()
 	if err != nil {
 		return err
 	}
@@ -1023,13 +1070,13 @@ func (t *transferQueueActiveTaskExecutor) updateWorkflowExecution(
 
 	if createWorkflowTask {
 		// Create a transfer task to schedule a workflow task
-		err := scheduleWorkflowTask(mutableState)
+		err := workflow.ScheduleWorkflowTask(mutableState)
 		if err != nil {
 			return err
 		}
 	}
 
-	return context.updateWorkflowExecutionAsActive(t.shard.GetTimeSource().Now())
+	return context.UpdateWorkflowExecutionAsActive(t.shard.GetTimeSource().Now())
 }
 
 func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionWithRetry(
@@ -1046,7 +1093,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionWithRetr
 				WorkflowId: task.GetTargetWorkflowId(),
 				RunId:      task.GetTargetRunId(),
 			},
-			Identity: identityHistoryService,
+			Identity: consts.IdentityHistoryService,
 			// Use the same request ID to dedupe RequestCancelWorkflowExecution calls
 			RequestId: requestCancelInfo.GetCancelRequestId(),
 		},
@@ -1065,7 +1112,7 @@ func (t *transferQueueActiveTaskExecutor) requestCancelExternalExecutionWithRetr
 		return err
 	}
 
-	err := backoff.Retry(op, persistenceOperationRetryPolicy, common.IsPersistenceTransientError)
+	err := backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
 	return err
 }
 
@@ -1083,7 +1130,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionWithRetry(
 				WorkflowId: task.GetTargetWorkflowId(),
 				RunId:      task.GetTargetRunId(),
 			},
-			Identity:   identityHistoryService,
+			Identity:   consts.IdentityHistoryService,
 			SignalName: signalInfo.Name,
 			Input:      signalInfo.Input,
 			// Use same request ID to deduplicate SignalWorkflowExecution calls
@@ -1104,7 +1151,7 @@ func (t *transferQueueActiveTaskExecutor) signalExternalExecutionWithRetry(
 		return err
 	}
 
-	return backoff.Retry(op, persistenceOperationRetryPolicy, common.IsPersistenceTransientError)
+	return backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
 }
 
 func (t *transferQueueActiveTaskExecutor) startWorkflowWithRetry(
@@ -1156,7 +1203,7 @@ func (t *transferQueueActiveTaskExecutor) startWorkflowWithRetry(
 		return err
 	}
 
-	err = backoff.Retry(op, persistenceOperationRetryPolicy, common.IsPersistenceTransientError)
+	err = backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
 	if err != nil {
 		return "", err
 	}
@@ -1168,10 +1215,10 @@ func (t *transferQueueActiveTaskExecutor) resetWorkflow(
 	namespace string,
 	reason string,
 	resetPoint *workflowpb.ResetPointInfo,
-	baseContext workflowExecutionContext,
-	baseMutableState mutableState,
-	currentContext workflowExecutionContext,
-	currentMutableState mutableState,
+	baseContext workflow.Context,
+	baseMutableState workflow.MutableState,
+	currentContext workflow.Context,
+	currentMutableState workflow.MutableState,
 	logger log.Logger,
 ) error {
 
@@ -1214,10 +1261,11 @@ func (t *transferQueueActiveTaskExecutor) resetWorkflow(
 			t.shard.GetClusterMetadata(),
 			currentContext,
 			currentMutableState,
-			noopReleaseFn, // this is fine since caller will defer on release
+			workflow.NoopReleaseFn, // this is fine since caller will defer on release
 		),
 		reason,
 		nil,
+		enumspb.RESET_REAPPLY_TYPE_SIGNAL,
 	)
 
 	switch err.(type) {
@@ -1320,7 +1368,7 @@ func (t *transferQueueActiveTaskExecutor) applyParentClosePolicy(
 				// If the child does continue as new it still propagates the RunID of first execution.
 				FirstExecutionRunId: childInfo.StartedRunId,
 				Reason:              "by parent close policy",
-				Identity:            identityHistoryService,
+				Identity:            consts.IdentityHistoryService,
 			},
 		})
 		return err
@@ -1336,7 +1384,7 @@ func (t *transferQueueActiveTaskExecutor) applyParentClosePolicy(
 				// Include StartedRunID as FirstExecutionRunID on the request to allow child to be canceled across runs.
 				// If the child does continue as new it still propagates the RunID of first execution.
 				FirstExecutionRunId: childInfo.StartedRunId,
-				Identity:            identityHistoryService,
+				Identity:            consts.IdentityHistoryService,
 			},
 		})
 		return err
