@@ -46,6 +46,7 @@ import (
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
+	"go.temporal.io/server/service/history/workflow"
 )
 
 type (
@@ -59,9 +60,9 @@ type (
 		hasBufferedEvents               bool
 		workflowTaskFailedCause         *workflowTaskFailedCause
 		activityNotStartedCancelled     bool
-		continueAsNewBuilder            mutableState
+		continueAsNewBuilder            workflow.MutableState
 		stopProcessing                  bool // should stop processing any more commands
-		mutableState                    mutableState
+		mutableState                    workflow.MutableState
 		initiatedChildExecutionsInBatch map[string]struct{} // Set of initiated child executions in the workflow task
 
 		// validation
@@ -83,7 +84,7 @@ type (
 func newWorkflowTaskHandler(
 	identity string,
 	workflowTaskCompletedID int64,
-	mutableState mutableState,
+	mutableState workflow.MutableState,
 	attrValidator *commandAttrValidator,
 	sizeLimitChecker *workflowSizeChecker,
 	logger log.Logger,
@@ -662,6 +663,16 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 		return err
 	}
 
+	failWorkflow, err = handler.sizeLimitChecker.failWorkflowIfMemoSizeExceedsLimit(
+		metrics.CommandTypeTag(enumspb.COMMAND_TYPE_CONTINUE_AS_NEW_WORKFLOW_EXECUTION.String()),
+		attr.GetMemo().Size(),
+		"ContinueAsNewWorkflowExecutionCommandAttributes. Memo exceeds size limit.",
+	)
+	if err != nil || failWorkflow {
+		handler.stopProcessing = true
+		return err
+	}
+
 	failWorkflow, err = handler.sizeLimitChecker.failWorkflowIfSearchAttributesSizeExceedsLimit(
 		attr.GetSearchAttributes(),
 		namespace,
@@ -756,7 +767,17 @@ func (handler *workflowTaskHandlerImpl) handleCommandStartChildWorkflow(
 	failWorkflow, err := handler.sizeLimitChecker.failWorkflowIfPayloadSizeExceedsLimit(
 		metrics.CommandTypeTag(enumspb.COMMAND_TYPE_START_CHILD_WORKFLOW_EXECUTION.String()),
 		attr.GetInput().Size(),
-		"StartChildWorkflowExecutionCommandAttributes.Input exceeds size limit.",
+		"StartChildWorkflowExecutionCommandAttributes. Input exceeds size limit.",
+	)
+	if err != nil || failWorkflow {
+		handler.stopProcessing = true
+		return err
+	}
+
+	failWorkflow, err = handler.sizeLimitChecker.failWorkflowIfMemoSizeExceedsLimit(
+		metrics.CommandTypeTag(enumspb.COMMAND_TYPE_START_CHILD_WORKFLOW_EXECUTION.String()),
+		attr.GetMemo().Size(),
+		"StartChildWorkflowExecutionCommandAttributes. Memo exceeds size limit.",
 	)
 	if err != nil || failWorkflow {
 		handler.stopProcessing = true
